Version for gpsdrive-1.31

Since version 1.29 GpsDrive supports SQL database support.

Supported databases (October 2002): MySQL

Compiling:
----------
MySQL support may only compiled, if you have installed the
mysql-client development package and can only be used if you have
libmysqlclient.so.

If you want to disable MySQL support at all, even if you have the
header and libs installed, call configure with the '--disable-mysql'
option.

Running and first initialization:
---------------------------------
If you have not already done, install the mysql server package for
your distribution.

o Start the server, mostly with: /etc/init.d/mysql start

o To create a initial database use to included SQL file 'create.sql'.
  Make sure, that the user  have enough permissions to create this table
  and grant the permissions! 

  mysql < create.sql
  (add username and login options if needed)
  i.e.:
  mysql -u root -p <create.sql

  If you have no password for root (this is the case if you newly
  installed mysql) so use this command:

  mysql -u root <create.sql

  You may change the string gast@localhost to gast@'%' if you want to
  use the database from other hosts.


Update:
-------
If you have already the database and you update GpsDrive, make sure
your table structure is the same like in create.sql.

Managing the SQL server:
------------------------

A good tool is mysqlcc, download it from:

http://www.mysql.com/downloads/gui-mycc.html

Security:
---------
GpsDrive uses the username 'gast' and the password 'gast' as default
to access the table 'waypoints' in the database 'geoinfo'. If you want to
change this in the SQL database, so change this in GpsDrive also. For
this, edit the file $HOME/.gpsdrive/gpsdriverc (dbuser,dbpass).

Be aware that the MySQL password for this database is stored in
cleartext in gpsdriverc!!!


Importing existing waypoint files:
----------------------------------
I also provided the script 'wp2sql', which creates the .sql input file
from gpsdrive waypointfiles. 

Change it for your needs first!!!!

Then do 

mysql -u gast -pgast <wp.sql

You can also import ASCII files with the 'mysqlimport' program.

=======================================================================
IMPORTANT:

Select the 'Use SQL' button to use the SQL generated waypoints.

=======================================================================

Waypoint types:
---------------

You can define freely your waypoint type names. But there are special
names, which results in special icons display in GpsDrive:

WLAN	     : open WLAN accesspoint
WLAN-WEP     : crypted WLAN accesspoint
Rest	     : a restaurant
McDonalds    : a kind of restaurant
BurgerKing   : another kind of restaurant
Hotel	     : A hotel
Shop	     : Shop
Monu	     : Monument, touristic attraction
Speedtrap    : Speedtrap (Radar), gives warning if this is in front of
	       you
Nightclub    : Pssst...
Airport	     : Airport
Golf	     : Golf course
Gasstation   : Gasstation
Cafe	     : coffee bar

The names are case insensitive.


